import requests
from typing import Any

from abc import ABC, abstractmethod


class BaseAPIClient(ABC):
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://api.curseforge.com",
        client: requests.Session | None = None,
    ):
        self.api_key = api_key
        self.base_url = base_url
        self.client = client or requests.Session()

    @abstractmethod
    def get(self, endpoint: str, params: dict[str, Any] | None = None) -> Any:
        """Send a GET request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the request to.
            params (dict[str, Any] | None, optional): The parameters to send with the request. Defaults to None.

        Returns:
            Any: The response data.
        """
        pass

    @abstractmethod
    def post(self, endpoint: str, data: dict[str, Any] | None = None) -> Any:
        """Send a POST request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the request to.
            data (dict[str, Any] | None, optional): The data to send with the request. Defaults to None.

        Returns:
            Any: The response data.
        """
        pass



from datetime import datetime

from pydantic import BaseModel, Field


class Category(BaseModel):
    """https://docs.curseforge.com/#tocS_Category"""

    id: int = Field(alias="id", description="The category id")
    game_id: int = Field(
        alias="gameId", description="The game id related to the category"
    )
    name: str = Field(alias="name", description="Category name")
    slug: str = Field(
        alias="slug", description="The category slug as it appears in the URL"
    )
    url: str = Field(alias="url", description="The category URL")
    icon_url: str = Field(alias="iconUrl", description="URL for the category icon")
    date_modified: datetime = Field(
        alias="dateModified", description="Last modified date of the category"
    )
    is_class: bool | None = Field(
        alias="isClass",
        description="A top level category for other categories",
        default=None,
    )
    class_id: int | None = Field(
        alias="classId",
        description="The class id of the category, meaning - the class of which this category is under",
        default=None,
    )
    parent_category_id: int | None = Field(
        alias="parentCategoryId",
        description="The parent category for this category",
        default=None,
    )
    display_index: int | None = Field(
        alias="displayIndex",
        description="The display index for this category",
        default=None,
    )



import requests
from typing import Any

from .base import BaseAPIClient
from .v1 import API_v1
from .v2 import API_v2


class APIClient(BaseAPIClient):
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://api.curseforge.com",
        client: requests.Session | None = None,
    ):
        self.api_key = api_key
        self.base_url = base_url
        self.client = self._init_client(client)

        self.v1 = API_v1(self)
        self.v2 = API_v2(self)

    def _init_client(self, client: requests.Session | None) -> requests.Session:
        """Initialize the client with the specified client or create a new one if none is provided.

        Args:
            client (requests.Session | None, optional): The client to use. Defaults to None.

        Returns:
            requests.Session: The initialized client.
        """
        _client = client or requests.Session()
        _client.headers.update(
            {
                "x-api-key": self.api_key,
                "Accept": "application/json",
            }
        )
        return _client

    def _build_request_uri(self, endpoint: str) -> str:
        """Build the request URI for the specified endpoint.

        Args:
            endpoint (str): The endpoint to build the URI for.

        Returns:
            str: The built URI.
        """
        return "{}/{}".format(self.base_url, endpoint.strip("/"))

    def get(self, endpoint: str, params: dict[str, Any] | None = None) -> Any:
        """Send a GET request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the request to.
            params (dict[str, Any] | None, optional): The parameters to send with the request. Defaults to None.

        Returns:
            Any: The response data.
        """
        response = self.client.get(
            url=self._build_request_uri(endpoint),
            params=params or {},
        )

        return response.json()

    def post(self, endpoint: str, data: dict[str, Any] | None = None) -> Any:
        """Send a POST request to the specified endpoint.

        Args:
            endpoint (str): The endpoint to send the request to.
            data (dict[str, Any] | None, optional): The data to send with the request. Defaults to None.

        Returns:
            Any: The response data.
        """
        response = self.client.post(
            url=self._build_request_uri(endpoint),
            data=data or {},
        )

        return response.json()



from datetime import datetime

from pydantic import BaseModel, Field

from .enums import (
    GameVersionStatus,
    GameVersionTypeStatus,
    ModLoaderInstallMethod,
    ModLoaderType,
)


class MinecraftGameVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_MinecraftGameVersion"""

    id: int = Field(alias="id")
    game_version_id: int = Field(alias="gameVersionId")
    version_string: str = Field(alias="versionString")
    jar_download_url: str = Field(alias="jarDownloadUrl")
    json_download_url: str = Field(alias="jsonDownloadUrl")
    approved: bool = Field(alias="approved")
    date_modified: datetime = Field(alias="dateModified")
    game_version_type_id: int = Field(alias="gameVersionTypeId")
    game_version_status: GameVersionStatus = Field(alias="gameVersionStatus")
    game_version_type_status: GameVersionTypeStatus = Field(
        alias="gameVersionTypeStatus"
    )


class MinecraftModLoaderIndex(BaseModel):
    """https://docs.curseforge.com/#tocS_MinecraftModLoaderIndex"""

    name: str = Field(alias="name")
    game_version: str = Field(alias="gameVersion")
    latest: bool = Field(alias="latest")
    recommended: bool = Field(alias="recommended")
    date_modified: datetime = Field(alias="dateModified")
    type: ModLoaderType = Field(alias="type")


class MinecraftModLoaderVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_MinecraftModLoaderVersion"""

    id: int = Field(alias="id")
    game_version_id: int = Field(alias="gameVersionId")
    minecraft_game_version_id: int = Field(alias="minecraftGameVersionId")
    forge_version: str = Field(alias="forgeVersion")
    name: str = Field(alias="name")
    type: ModLoaderType = Field(alias="type")
    download_url: str = Field(alias="downloadUrl")
    filename: str = Field(alias="filename")
    install_method: ModLoaderInstallMethod = Field(alias="installMethod")
    latest: bool = Field(alias="latest")
    recommended: bool = Field(alias="recommended")
    approved: bool = Field(alias="approved")
    date_modified: datetime = Field(alias="dateModified")
    maven_version_string: str = Field(alias="mavenVersionString")
    version_json: str = Field(alias="versionJson")
    libraries_install_location: str = Field(alias="librariesInstallLocation")
    minecraft_version: str = Field(alias="minecraftVersion")
    additional_files_json: str = Field(alias="additionalFilesJson")
    mod_loader_game_version_id: int = Field(alias="modLoaderGameVersionId")
    mod_loader_game_version_type_id: int = Field(alias="modLoaderGameVersionTypeId")
    mod_loader_game_version_status: GameVersionStatus = Field(
        alias="modLoaderGameVersionStatus"
    )
    mod_loader_game_version_type_status: GameVersionTypeStatus = Field(
        alias="modLoaderGameVersionTypeStatus"
    )
    mc_game_version_id: int = Field(alias="mcGameVersionId")
    mc_game_version_type_id: int = Field(alias="mcGameVersionTypeId")
    mc_game_version_status: GameVersionStatus = Field(alias="mcGameVersionStatus")
    mc_game_version_type_status: GameVersionTypeStatus = Field(
        alias="mcGameVersionTypeStatus"
    )



from pydantic import BaseModel, Field

from .files import File


class FingerprintMatch(BaseModel):
    """https://docs.curseforge.com/#tocS_FingerprintMatch"""

    id: int = Field(alias="id")
    file: File = Field(alias="file")
    latest_files: list[File] = Field(alias="latestFiles")


class FingerprintMatchesResult(BaseModel):
    """https://docs.curseforge.com/#tocS_FingerprintsMatchesResult"""

    is_cache_built: bool = Field(alias="isCacheBuilt")
    exact_matches: list[FingerprintMatch] = Field(alias="exactMatches")
    exact_fingerprints: list[int] = Field(alias="exactFingerprints")
    partial_matches: list[FingerprintMatch] = Field(alias="partialMatches")
    partial_match_fingerprints: dict[str, list[int]] = Field(
        alias="partialMatchFingerprints"
    )
    additional_properties: list[int] = Field(alias="additionalProperties")
    installed_fingerprints: list[int] = Field(alias="installedFingerprints")
    unmatched_fingerprints: list[int] = Field(alias="unmatchedFingerprints")


class FolderFingerprint(BaseModel):
    """https://docs.curseforge.com/#tocS_FolderFingerprint"""

    foldername: str = Field(alias="foldername")
    fingerprints: list[int] = Field(alias="fingerprints")


class FingerprintFuzzyMatch(BaseModel):
    """https://docs.curseforge.com/#tocS_FingerprintFuzzyMatch"""

    id: int = Field(alias="id")
    file: File = Field(alias="file")
    latest_files: list[File] = Field(alias="latestFiles")
    fingerprints: list[int] = Field(alias="fingerprints")


class FingerprintFuzzyMatchResult(BaseModel):
    """https://docs.curseforge.com/#tocS_FingerprintFuzzyMatchResult"""

    fuzzyMatches: list[FingerprintMatch] = Field(alias="fuzzyMatches")



from datetime import datetime

from pydantic import BaseModel, Field

from .category import Category
from .enums import ModStatus
from .files import File, FileIndex


class ModAsset(BaseModel):
    """https://docs.curseforge.com/#tocS_ModAsset"""

    id: int = Field(alias="id")
    mod_id: int = Field(alias="modId")
    title: str = Field(alias="title")
    description: str = Field(alias="description")
    thumbnail_url: str = Field(alias="thumbnailUrl")
    url: str = Field(alias="url")


class ModAuthor(BaseModel):
    """https://docs.curseforge.com/#tocS_ModAuthor"""

    id: int = Field(alias="id")
    name: str = Field(alias="name")
    url: str = Field(alias="url")


class ModLinks(BaseModel):
    """https://docs.curseforge.com/#tocS_ModLinks"""

    website_url: str | None = Field(alias="websiteUrl")
    wiki_url: str | None = Field(alias="wikiUrl")
    issues_url: str | None = Field(alias="issuesUrl")
    source_url: str | None = Field(alias="sourceUrl")


class Mod(BaseModel):
    """https://docs.curseforge.com/#tocS_Mod"""

    id: int = Field(alias="id", description="The mod id")
    game_id: int = Field(alias="gameId", description="The game id this mod is for")
    name: str = Field(alias="name", description="The name of the mod")
    slug: str = Field(
        alias="slug", description="The mod slug that would appear in the URL"
    )
    links: ModLinks = Field(
        alias="links",
        description="Relevant links for the mod such as Issue tracker and Wiki",
    )
    summary: str = Field(alias="summary", description="Mod summary")
    status: ModStatus = Field(alias="status", description="Current mod status")
    download_count: int = Field(
        alias="downloadCount", description="Number of downloads for the mod"
    )
    is_featured: bool = Field(
        alias="isFeatured",
        description="Whether the mod is included in the featured mods list",
    )
    primary_category_id: int = Field(
        alias="primaryCategoryId",
        description="The main category of the mod as it was chosen by the mod author",
    )
    categories: list[Category] = Field(
        alias="categories", description="List of categories that this mod is related to"
    )
    class_id: int = Field(
        alias="classId", description="The class id this mod belongs to"
    )
    authors: list[ModAuthor] = Field(
        alias="authors", description="List of the mod's authors"
    )
    logo: ModAsset | None = Field(alias="logo", description="The mod's logo asset", default=None)
    screenshots: list[ModAsset] = Field(
        alias="screenshots", description="List of screenshots assets"
    )
    main_file_id: int = Field(
        alias="mainFileId", description="The id of the main file of the mod"
    )
    latest_files: list[File] = Field(
        alias="latestFiles", description="List of latest files of the mod"
    )
    latest_files_indexes: list[FileIndex] = Field(
        alias="latestFilesIndexes",
        description="List of file related details for the latest files of the mod",
    )
    latest_early_access_files_indexes: list[FileIndex] = Field(
        alias="latestEarlyAccessFilesIndexes",
        description="List of file related details for the latest early access files of the mod",
    )
    date_created: datetime = Field(
        alias="dateCreated", description="The creation date of the mod"
    )
    date_modified: datetime = Field(
        alias="dateModified", description="The last time the mod was modified"
    )
    date_released: datetime = Field(
        alias="dateReleased", description="The release date of the mod"
    )
    allow_mod_distribution: bool | None = Field(
        alias="allowModDistribution",
        description="Is mod allowed to be distributed",
        default=None,
    )
    game_popularity_rank: int = Field(
        alias="gamePopularityRank", description="The mod popularity rank for the game"
    )
    is_available: bool = Field(
        alias="isAvailable",
        description="Is the mod available for search. This can be false when a mod is experimental, in a deleted state or has only alpha files",
    )
    thumbs_up_count: int = Field(
        alias="thumbsUpCount", description="The mod's thumbs up count"
    )
    rating: float | None = Field(
        alias="rating", description="The mod's Rating", default=None
    )



from cursedforged.api.base import BaseAPIClient

from cursedforged.types import GetVersionsResponse2


class API_v2:
    def __init__(self, client: BaseAPIClient):
        self.client = client

    def get_game_versions(self, game_id: int) -> GetVersionsResponse2:
        """Get all available versions for each known version type of the specified game. A private game is only accessible to its respective API key.

        https://docs.curseforge.com/#get-versions-v2

        Args:
            game_id (int): A game unique id

        Returns:
            GetVersionsResponse2: A response object
        """
        response = self.client.get(
            "v2/games/{}/versions".format(game_id),
        )

        return GetVersionsResponse2(**response)



from datetime import datetime

from pydantic import BaseModel, Field

from .enums import (
    FileRelationType,
    FileReleaseType,
    FileStatus,
    HashAlgo,
    ModLoaderType,
)


class FileDependency(BaseModel):
    """https://docs.curseforge.com/#tocS_FileDependency"""

    mod_id: int = Field(alias="modId")
    relation_type: FileRelationType = Field(alias="relationType")


class FileHash(BaseModel):
    """https://docs.curseforge.com/#tocS_FileHash"""

    value: str = Field(alias="value")
    algo: HashAlgo = Field(alias="algo")


class FileIndex(BaseModel):
    """https://docs.curseforge.com/#tocS_FileIndex"""

    game_version: str = Field(alias="gameVersion")
    file_id: int = Field(alias="fileId")
    filename: str = Field(alias="filename")
    release_type: FileReleaseType = Field(alias="releaseType")
    game_version_type_id: int | None = Field(alias="gameVersionTypeId", default=None)
    mod_loader: ModLoaderType | None = Field(alias="modLoader", default=None)


class FileModule(BaseModel):
    """https://docs.curseforge.com/#tocS_FileModule"""

    name: str = Field(alias="name")
    fingerprint: int = Field(alias="fingerprint")


class SortableGameVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_SortableGameVersion"""

    game_version_name: str = Field(
        alias="gameVersionName", description="Original version name (e.g. 1.5b)"
    )
    game_version_padded: str = Field(
        alias="gameVersionPadded",
        description="Used for sorting (e.g. 0000000001.0000000005)",
    )
    game_version: str = Field(
        alias="gameVersion", description="Game version clean name (e.g. 1.5)"
    )
    game_version_release_date: datetime = Field(
        alias="gameVersionReleaseDate", description="Game version release date"
    )
    game_version_type_id: int | None = Field(
        alias="gameVersionTypeId", description="Game version type id", default=None
    )


class File(BaseModel):
    """https://docs.curseforge.com/#tocS_File"""

    id: int = Field(alias="id", description="The file id")
    game_id: int = Field(
        alias="gameId",
        description="The game id related to the mod that this file belongs to",
    )
    mod_id: int = Field(alias="modId", description="The mod id")
    is_available: bool = Field(
        alias="isAvailable", description="Whether the file is available to download"
    )
    display_name: str = Field(
        alias="displayName", description="Display name of the file"
    )
    file_name: str = Field(alias="fileName", description="Exact file name")
    release_type: FileReleaseType = Field(
        alias="releaseType", description="The file release type"
    )
    file_status: FileStatus = Field(
        alias="fileStatus", description="Status of the file"
    )
    hashes: list[FileHash] = Field(
        alias="hashes", description="The file hash (i.e. md5 or sha1)"
    )
    file_date: datetime = Field(alias="fileDate", description="The file timestamp")
    file_length: int = Field(alias="fileLength", description="The file length in bytes")
    download_count: int = Field(
        alias="downloadCount", description="The number of downloads for the file"
    )
    file_size_on_disk: int | None = Field(
        alias="fileSizeOnDisk", description="The file's size on disk", default=None
    )
    download_url: str | None = Field(alias="downloadUrl", description="The file download URL", default=None)
    game_versions: list[str] = Field(
        alias="gameVersions",
        description="List of game versions this file is relevant for",
    )
    sortable_game_versions: list[SortableGameVersion] = Field(
        alias="sortableGameVersions",
        description="Metadata used for sorting by game versions",
    )
    dependencies: list[FileDependency] = Field(
        alias="dependencies", description="List of dependencies files"
    )
    expose_as_alternative: bool | None = Field(
        alias="exposeAsAlternative", default=None
    )
    parent_project_file_id: int | None = Field(
        alias="parentProjectFileId", default=None
    )
    alternate_file_id: int | None = Field(alias="alternateFileId", default=None)
    is_server_pack: bool | None = Field(alias="isServerPack", default=None)
    server_pack_file_id: int | None = Field(alias="serverPackFileId", default=None)
    is_early_access_content: bool | None = Field(
        alias="isEarlyAccessContent", default=None
    )
    early_access_end_date: datetime | None = Field(
        alias="earlyAccessEndDate", default=None
    )
    file_fingerprint: int = Field(alias="fileFingerprint")
    modules: list[FileModule] = Field(alias="modules")



from ..base import BaseAPIClient

from cursedforged.types import (
    Game,
    ModsSearchSortField,
    SortOrder,
    ModLoaderType,
    GetGamesResponse,
    GetVersionsResponse,
    GetVersionTypesResponse,
    GetCategoriesResponse,
    GetModResponse,
    GetModsResponse,
    GetFeaturedModsResponse,
    StringResponse,
    GetModFileResponse,
    GetModFilesResponse,
    GetFilesResponse,
    ApiResponseOfListOfMinecraftGameVersion,
    ApiResponseOfMinecraftGameVersion,
    ApiResponseOfListOfMinecraftModLoaderIndex,
    ApiResponseOfMinecraftModLoaderVersion,
    GetFingerprintMatchesResponse,
    FolderFingerprint,
    GetFingerprintsFuzzyMatchesResponse,
)


class API_v1:
    def __init__(self, client: BaseAPIClient):
        self.client = client

    def get_games(self, index: int = 0, page_size: int = 50) -> GetGamesResponse:
        """Get all games

        https://docs.curseforge.com/#get-games

        Args:
            index (int, optional): A zero based index of the first item to include in the response, the limit is: (index + pageSize <= 10,000).
            page_size (int, optional): The number of items to include in the response, the default/maximum value is 50.

        Returns:
            GetGamesResponse: A response object
        """
        response = self.client.get(
            "v1/games",
            params={
                "index": index,
                "pageSize": page_size,
            },
        )

        return GetGamesResponse(**response)

    def get_game(self, game_id: int) -> Game:
        """Get a game

        https://docs.curseforge.com/#get-game

        Args:
            game_id (int): A game unique id

        Returns:
            Game: A game object
        """
        response = self.client.get(
            "v1/games/{}".format(game_id),
        )
        return Game(**dict(response).get("data", {}))

    def get_game_versions(self, game_id: int) -> GetVersionsResponse:
        """Get all available versions for each known version type of the specified game. A private game is only accessible to its respective API key.

        https://docs.curseforge.com/#get-versions

        Args:
            game_id (int): A game unique id

        Returns:
            GetVersionsResponse: A response object
        """
        response = self.client.get(
            "v1/games/{}/versions".format(game_id),
        )

        return GetVersionsResponse(**response)

    def get_game_version_types(self, game_id: int) -> GetVersionTypesResponse:
        """Get all available version types of the specified game.

        https://docs.curseforge.com/#get-version-types

        A private game is only accessible to its respective API key.

        Currently, when creating games via the CurseForge for Studios Console, you are limited to a single game version type. This means that this endpoint is probably not useful in most cases and is relevant mostly when handling existing games that have multiple game versions such as World of Warcraft and Minecraft (e.g. 517 for wow_retail).

        Args:
            game_id (int): A game unique id

        Returns:
            GetVersionTypesResponse: A response object
        """
        response = self.client.get(
            "v1/games/{}/version-types".format(game_id),
        )
        return GetVersionTypesResponse(**response)

    def get_categories(
        self, game_id: int, class_id: int | None = None, classes_only: bool = False
    ) -> GetCategoriesResponse:
        """Get all available classes and categories of the specified game. Specify a game id for a list of all game categories, or a class id for a list of categories under that class. specifiy the classes Only flag to just get the classes for a given game.

        https://docs.curseforge.com/#get-categories

        Args:
            game_id (int): A game unique id
            class_id (int | None, optional): A class unique id. Defaults to None.
            classes_only (bool, optional): If true, only return the classes for the specified game. Defaults to False.

        Returns:
            GetCategoriesResponse: A response object
        """

        response = self.client.get(
            "v1/categories",
            params={
                "gameId": game_id,
                "classId": class_id,
                "classesOnly": classes_only,
            },
        )
        return GetCategoriesResponse(**response)

    def search_mods(
        self,
        game_id: int,
        class_id: int | None = None,
        category_id: int | None = None,
        category_ids: list[int] | None = None,
        game_version: str | None = None,
        game_versions: list[str] | None = None,
        search_filter: str | None = None,
        sort_field: ModsSearchSortField | None = None,
        sort_order: SortOrder | None = None,
        mod_loader_type: ModLoaderType | None = None,
        mod_loader_types: list[ModLoaderType] | None = None,
        game_version_type_id: int | None = None,
        author_id: int | None = None,
        primary_author_id: int | None = None,
        slug: str | None = None,
        index: int | None = 0,
        page_size: int | None = 50,
    ):
        """Get all mods that match the search criteria.

        https://docs.curseforge.com/#search-mods

        Args:
            game_id (int): A game unique id
            class_id (int | None, optional): Filter by section id (discoverable via Categories)
            category_id (int | None, optional): Filter by category id
            category_ids (list[int] | None, optional): Filter by a list of category ids - this will override categoryId
            game_version (str | None, optional): Filter by game version string
            game_versions (list[str] | None, optional): Filter by a list of game version strings - this will override
            search_filter (str | None, optional): Filter by free text search in the mod name and author
            sort_field (ModsSearchSortField | None, optional): Filter by ModsSearchSortField enumeration
            sort_order (SortOrder | None, optional): 'asc' if sort is in ascending order, 'desc' if sort is in descending order
            mod_loader_type (ModLoaderType | None, optional): Filter only mods associated to a given modloader (Forge, Fabric ...). Must be coupled with gameVersion.
            mod_loader_types (list[ModLoaderType] | None, optional): Filter by a list of mod loader types - this will override modLoaderType
            game_version_type_id (int | None, optional): Filter only mods that contain files tagged with versions of the given gameVersionTypeId
            author_id (int | None, optional): Filter only mods that the given authorId is a member of.
            primary_author_id (int | None, optional): Filter only mods that the given primaryAuthorId is the owner of.
            slug (str | None, optional): Filter by slug (coupled with classId will result in a unique result).
            index (int | None, optional): A zero based index of the first item to include in the response, the limit is: (index + pageSize <= 10,000).
            page_size (int | None, optional): The number of items to include in the response, the default/maximum value is 50.

        Returns:
            GetCategoriesResponse: A response object
        """
        response = self.client.get(
            "v1/mods/search",
            params={
                "gameId": game_id,
                "classId": class_id,
                "categoryId": category_id,
                "categoryIds": category_ids,
                "gameVersion": game_version,
                "gameVersions": game_versions,
                "searchFilter": search_filter,
                "sortField": sort_field,
                "sortOrder": sort_order,
                "modLoaderType": mod_loader_type,
                "modLoaderTypes": mod_loader_types,
                "gameVersionTypeId": game_version_type_id,
                "authorId": author_id,
                "primaryAuthorId": primary_author_id,
                "slug": slug,
                "index": index,
                "pageSize": page_size,
            },
        )
        return GetCategoriesResponse(**response)

    def get_mod(self, mod_id: int) -> GetModResponse:
        """Get a single mod.

        https://docs.curseforge.com/#get-mod

        Args:
            mod_id (int): The mod id

        Returns:
            GetModResponse: A response object
        """
        response = self.client.get(
            "v1/mods/{}".format(mod_id),
        )
        return GetModResponse(**response)

    def get_mods(
        self, mod_ids: list[int], filter_pc_only: bool | None = False
    ) -> GetModsResponse:
        """Get a list of mods belonging the the same game.

        https://docs.curseforge.com/#get-mods

        Args:
            mod_ids (list[int]): A list of mod ids
            filter_pc_only (bool | None, optional): Filter mods that are only available for PC. Defaults to False.

        Returns:
            GetModsResponse: A response object
        """
        response = self.client.post(
            "v1/mods",
            data={
                "modIds": mod_ids,
                "filterPcOnly": filter_pc_only,
            },
        )
        return GetModsResponse(**response)

    def get_featured_mods(
        self,
        game_id: int,
        excluded_mod_ids: list[int] | None = None,
        game_version_type_id: int | None = None,
    ) -> GetFeaturedModsResponse:
        """Get a list of featured, popular and recently updated mods.

        https://docs.curseforge.com/#get-featured-mods

        Args:
            game_id (int): A game unique id
            excluded_mod_ids (list[int] | None, optional): A list of mod ids to exclude from the response. Defaults to None.
            game_version_type_id (int | None, optional): Filter mods by game version type. Defaults to None.

        Returns:
            GetFeaturedModsResponse: A response object
        """
        response = self.client.post(
            "v1/mods/featured",
            data={
                "gameId": game_id,
                "excludedModIds": excluded_mod_ids,
                "gameVersionTypeId": game_version_type_id,
            },
        )
        return GetFeaturedModsResponse(**response)

    def get_mod_description(
        self,
        mod_id: int,
        raw: bool | None = None,
        stripped: bool | None = None,
        markup: bool | None = None,
    ) -> StringResponse:
        """Get the full description of a mod in HTML format.

        https://docs.curseforge.com/#get-mod-description

        Args:
            mod_id (int): The mod id
            raw (bool | None, optional): If true, the description will be returned as it is stored in the database. Defaults to None.
            stripped (bool | None, optional): If true, the description will be stripped of all HTML tags. Defaults to None.
            markup (bool | None, optional): If true, the description will be returned as it is stored in the database. Defaults to None.

        Returns:
            StringResponse: The mod description
        """
        response = self.client.get(
            "v1/mods/{}/description".format(mod_id),
            params={
                "raw": raw,
                "stripped": stripped,
                "markup": markup,
            },
        )
        return StringResponse(**response)

    def get_mod_file(self, mod_id: int, file_id: int) -> GetModFileResponse:
        """Get a single file of the specified mod.

        https://docs.curseforge.com/#get-mod-file

        Args:
            mod_id (int): The mod id the file belongs to
            file_id (int): The file id

        Returns:
            GetModFileResponse: The mod file
        """
        response = self.client.get(
            "v1/mods/{}/files/{}".format(mod_id, file_id),
        )
        return GetModFileResponse(**response)

    def get_mod_files(
        self,
        mod_id: int,
        game_version: str | None = None,
        mod_loader_type: ModLoaderType | None = None,
        game_version_type_id: int | None = None,
        index: int | None = 0,
        page_size: int | None = 50,
    ) -> GetModFilesResponse:
        """Get all files of the specified mod.

        https://docs.curseforge.com/#get-mod-files

        Args:
            mod_id (int): The mod id the files belong to
            game_version (str | None, optional): Filter by game version string. Defaults to None.
            mod_loader_type (ModLoaderType | None, optional): ModLoaderType enumeration. Defaults to None.
            game_version_type_id (int | None, optional): Filter only files that are tagged with versions of the given gameVersionTypeId. Defaults to None.
            index (int | None, optional): A zero based index of the first item to include in the response, the limit is: (index + pageSize <= 10,000).
            page_size (int | None, optional): The number of items to include in the response, the default/maximum value is 50.

        Returns:
            GetModFilesResponse: A response object
        """
        response = self.client.get(
            "v1/mods/{}/files".format(mod_id),
            params={
                "gameVersion": game_version,
                "modLoaderType": mod_loader_type,
                "gameVersionTypeId": game_version_type_id,
                "index": index,
                "pageSize": page_size,
            },
        )
        return GetModFilesResponse(**response)

    def get_files(self, file_ids: list[int]) -> GetFilesResponse:
        """Get a list of files.

        https://docs.curseforge.com/#get-files

        Args:
            file_ids (list[int]): A list of file ids

        Returns:
            GetFilesResponse: A response object
        """
        response = self.client.post(
            "v1/mods/files",
            data={
                "fileIds": file_ids,
            },
        )
        return GetFilesResponse(**response)

    def get_mod_file_changelog(self, mod_id: int, file_id: int) -> StringResponse:
        """Get the changelog of a file in HTML format.

        https://docs.curseforge.com/#get-mod-file-changelog

        Args:
            mod_id (int): The mod id the file belongs to
            file_id (int): The file id

        Returns:
            StringResponse: A response object
        """
        response = self.client.get(
            "v1/mods/{}/files/{}/changelog".format(mod_id, file_id),
        )
        return StringResponse(**response)

    def get_mod_file_download_url(self, mod_id: int, file_id: int) -> StringResponse:
        """Get a download url for a specific file.

        https://docs.curseforge.com/#get-mod-file-download-url

        Args:
            mod_id (int): The mod id the file belongs to
            file_id (int): The file id

        Returns:
            StringResponse: A response object
        """
        response = self.client.get(
            "v1/mods/{}/files/{}/download-url".format(mod_id, file_id),
        )
        return StringResponse(**response)

    def get_fingerprints_matches_by_game_id(
        self, game_id: int, fingerprints: list[int]
    ) -> GetFingerprintMatchesResponse:
        """Get mod files that match a list of fingerprints for a given game id.

        https://docs.curseforge.com/#get-fingerprints-matches-by-game-id

        Args:
            game_id (int): The game id for matching fingerprints
            fingerprints (list[int]): The request body containing an array of fingerprints

        Returns:
            GetFingerprintMatchesResponse: A response object
        """
        response = self.client.post(
            "v1/fingerprints/{}".format(game_id),
            data={
                "fingerprints": fingerprints,
            },
        )
        return GetFingerprintMatchesResponse(**response)

    def get_fingerprints_matches(
        self, fingerprints: list[int]
    ) -> GetFingerprintMatchesResponse:
        """Get mod files that match a list of fingerprints.

        https://docs.curseforge.com/#get-fingerprints-matches

        Args:
            fingerprints (list[int]): The request body containing an array of fingerprints

        Returns:
            GetFingerprintMatchesResponse: A response object
        """
        response = self.client.post(
            "v1/fingerprints",
            data={
                "fingerprints": fingerprints,
            },
        )
        return GetFingerprintMatchesResponse(**response)

    def get_fingerprints_fuzzy_matches_by_game_id(
        self, game_id: int, fingerprints: list[FolderFingerprint]
    ) -> GetFingerprintsFuzzyMatchesResponse:
        """Get mod files that match a list of fingerprints using fuzzy matching.

        https://docs.curseforge.com/#get-fingerprints-fuzzy-matches-by-game-id

        Args:
            game_id (int): The game id for matching fingerprints
            fingerprints (list[FolderFingerprint]): Game id and folder fingerprints options for the fuzzy matching

        Returns:
            GetFingerprintsFuzzyMatchesResponse: A response object
        """
        response = self.client.post(
            "/v1/fingerprints/fuzzy/{}".format(game_id),
            data={
                "fingerprints": fingerprints,
            },
        )
        return GetFingerprintsFuzzyMatchesResponse(**response)

    def get_fingerprints_fuzzy_matches(
        self, fingerprints: list[FolderFingerprint]
    ) -> GetFingerprintsFuzzyMatchesResponse:
        """Get mod files that match a list of fingerprints using fuzzy matching.

        https://docs.curseforge.com/#get-fingerprints-fuzzy-matches

        Args:
            fingerprints (list[FolderFingerprint]): Game id and folder fingerprints options for the fuzzy matching

        Returns:
            GetFingerprintsFuzzyMatchesResponse: A response object
        """
        response = self.client.post(
            "/v1/fingerprints/fuzzy",
            data={
                "fingerprints": fingerprints,
            },
        )
        return GetFingerprintsFuzzyMatchesResponse(**response)

    def get_minecraft_versions(
        self, sort_descending: bool | None = None
    ) -> ApiResponseOfListOfMinecraftGameVersion:
        """Get a list of all available Minecraft versions.

        https://docs.curseforge.com/#get-minecraft-versions

        Args:
            sort_descending (bool, optional): If true, the list will be sorted in descending order. Defaults to None.

        Returns:
            ApiResponseOfListOfMinecraftGameVersion: A response object
        """
        response = self.client.get(
            "v1/minecraft/version",
            params={
                "sortDescending": sort_descending,
            },
        )
        return ApiResponseOfListOfMinecraftGameVersion(**response)

    def get_minecraft_version(
        self, game_version_string: str
    ) -> ApiResponseOfMinecraftGameVersion:
        """Get a single Minecraft version.

        https://docs.curseforge.com/#get-specific-minecraft-version

        Args:
            game_version_string (str): The game version string

        Returns:
            ApiResponseOfMinecraftGameVersion: A response object
        """
        response = self.client.get(
            "v1/minecraft/version/{}".format(game_version_string),
        )
        return ApiResponseOfMinecraftGameVersion(**response)

    def get_minecraft_modloaders(
        self, version: str | None = None, include_all: bool | None = None
    ) -> ApiResponseOfListOfMinecraftModLoaderIndex:
        """Get a list of all available Minecraft modloaders.

        https://docs.curseforge.com/#get-minecraft-modloaders

        Args:
            version (str | None, optional): Filter by game version string. Defaults to None.
            include_all (bool | None, optional): If true, include all modloaders. Defaults to None.

        Returns:
            ApiResponseOfListOfMinecraftModLoaderIndex: A response object
        """
        response = self.client.get(
            "v1/minecraft/modloader",
            params={
                "version": version,
                "includeAll": include_all,
            },
        )
        return ApiResponseOfListOfMinecraftModLoaderIndex(**response)

    def get_minecraft_modloader(
        self, mod_loader_name: str
    ) -> ApiResponseOfMinecraftModLoaderVersion:
        """Get a single Minecraft modloader.

        https://docs.curseforge.com/#get-specific-minecraft-modloader

        Args:
            modloader_id (str): The modloader name

        Returns:
            ApiResponseOfMinecraftModLoaderVersion: A response object
        """
        response = self.client.get(
            "v1/minecraft/modloader/{}".format(mod_loader_name),
        )
        return ApiResponseOfMinecraftModLoaderVersion(**response)



from datetime import datetime

from pydantic import BaseModel, Field

from .enums import CoreApiStatus, CoreStatus, GameVersionTypeStatus


class GameAssets(BaseModel):
    """https://docs.curseforge.com/#tocS_GameAssets"""

    icon_url: str | None = Field(alias="iconUrl")
    tile_url: str | None = Field(alias="tileUrl")
    cover_url: str | None = Field(alias="coverUrl")


class GameVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_GameVersion"""

    id: int = Field(alias="id")
    slug: str = Field(alias="slug")
    name: str = Field(alias="name")


class GameVersionsByType(BaseModel):
    """https://docs.curseforge.com/#tocS_GameVersionsByType"""

    type: int = Field(alias="type")
    versions: list[str] = Field(alias="versions")


class GameVersionsByType2(BaseModel):
    """https://docs.curseforge.com/#tocS_GameVersionsByType2"""

    type: int = Field(alias="type")
    versions: list[str] = Field(alias="versions")


class GameVersionType(BaseModel):
    """https://docs.curseforge.com/#tocS_GameVersionType"""

    id: int = Field(alias="id")
    game_id: int = Field(alias="gameId")
    name: str = Field(alias="name")
    slug: str = Field(alias="slug")
    is_syncable: bool = Field(alias="isSyncable")
    status: GameVersionTypeStatus = Field(alias="status")


class Game(BaseModel):
    """https://docs.curseforge.com/#tocS_Game"""

    id: int = Field(alias="id")
    name: str = Field(alias="name")
    slug: str = Field(alias="slug")
    date_modified: datetime = Field(alias="dateModified")
    assets: GameAssets = Field(alias="assets")
    status: CoreStatus = Field(alias="status")
    api_status: CoreApiStatus = Field(alias="apiStatus")



from enum import IntEnum, StrEnum


class CoreApiStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_CoreApiStatus"""

    PRIVATE = 1
    PUBLIC = 2


class CoreStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_CoreStatus"""

    DRAFT = 1
    TEST = 2
    PENDING_REVIEW = 3
    REJECTED = 4
    APPROVED = 5
    LIVE = 6


class GameVersionStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_GameVersionStatus"""

    APPROVED = 1
    DELETED = 2
    NEW = 3


class GameVersionTypeStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_GameVersionTypeStatus"""

    NORMAL = 1
    DELETED = 2


class ModStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_ModStatus"""

    NEW = 1
    CHANGES_REQUIRED = 2
    UNDER_SOFT_REVIEW = 3
    APPROVED = 4
    REJECTED = 5
    CHANGES_MADE = 6
    INACTIVE = 7
    ABANDONED = 8
    DELETED = 9
    UNDER_REVIEW = 10


class ModsSearchSortField(IntEnum):
    """https://docs.curseforge.com/#tocS_ModsSearchSortField"""

    FEATURED = 1
    POPULARITY = 2
    LAST_UPDATED = 3
    NAME = 4
    AUTHOR = 5
    TOTAL_DOWNLOADS = 6
    CATEGORY = 7
    GAME_VERSION = 8
    EARLY_ACCESS = 9
    FEATURED_RELEASED = 10
    RELEASED_DATE = 11
    RATING = 12


class ModLoaderInstallMethod(IntEnum):
    """https://docs.curseforge.com/#tocS_ModLoaderInstallMethod"""

    FORGE_INSTALLER = 1
    FORGE_JAR_INSTALL = 2
    FORGE_INSTALLER_V2 = 3


class ModLoaderType(IntEnum):
    """https://docs.curseforge.com/#tocS_ModLoaderType"""

    ANY = 0
    FORGE = 1
    CAULDRON = 2
    LITELOADER = 3
    FABRIC = 4
    QUILT = 5
    NEOFORGE = 6


class FileRelationType(IntEnum):
    """https://docs.curseforge.com/#tocS_FileRelationType"""

    EMBEDDED_LIBRARY = 1
    OPTIONAL_DEPENDENCY = 2
    REQUIRED_DEPENDENCY = 3
    TOOL = 4
    INCOMPATIBLE = 5
    INCLUDE = 6


class FileReleaseType(IntEnum):
    """https://docs.curseforge.com/#tocS_FileReleaseType"""

    RELEASE = 1
    BETA = 2
    ALPHA = 3


class FileStatus(IntEnum):
    """https://docs.curseforge.com/#tocS_FileStatus"""

    PROCESSING = 1
    CHANGES_REQUIRED = 2
    UNDER_REVIEW = 3
    APPROVED = 4
    REJECTED = 5
    MALWARE_DETECTED = 6
    DELETED = 7
    ARCHIVED = 8
    TESTING = 9
    RELEASED = 10
    READY_FOR_REVIEW = 11
    DEPRECATED = 12
    BAKING = 13
    AWAITING_PUBLISHING = 14
    FAILED_PUBLISHING = 15


class HashAlgo(IntEnum):
    """https://docs.curseforge.com/#tocS_HashAlgo"""

    SHA1 = 1
    MD5 = 2


class SortOrder(StrEnum):
    """https://docs.curseforge.com/#tocS_SortOrder"""

    ASC = "asc"
    DESC = "desc"



from pydantic import BaseModel, Field

from .category import Category
from .minecraft import (
    MinecraftGameVersion,
    MinecraftModLoaderIndex,
    MinecraftModLoaderVersion,
)
from .mods import Mod
from .files import File
from .games import Game, GameVersionType, GameVersionsByType
from .fingerprints import FingerprintMatchesResult, FingerprintFuzzyMatchResult


class Pagination(BaseModel):
    """https://docs.curseforge.com/#tocS_Pagination"""

    index: int = Field(
        alias="index",
        description="A zero based index of the first item that is included in the response",
    )
    page_size: int = Field(
        alias="pageSize",
        description="The requested number of items to be included in the response",
    )
    result_count: int = Field(
        alias="resultCount",
        description="The actual number of items that were included in the response",
    )
    total_count: int = Field(
        alias="totalCount",
        description="The total number of items available by the request",
    )


class ApiResponseOfListOfMinecraftModLoaderIndex(BaseModel):
    """https://docs.curseforge.com/#tocS_ApiResponseOfListOfMinecraftModLoaderIndex"""

    data: list[MinecraftModLoaderIndex]


class ApiResponseOfMinecraftGameVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_ApiResponseOfMinecraftGameVersion"""

    data: MinecraftGameVersion


class ApiResponseOfMinecraftModLoaderVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_ApiResponseOfMinecraftModLoaderVersion"""

    data: MinecraftModLoaderVersion


class GetCategoriesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Categories%20Response"""

    data: list[Category]


class GetFeaturedModsResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Featured%20Mods%20Response"""

    featured: list[Mod] = Field(alias="featured")
    popular: list[Mod] = Field(alias="popular")
    recently_updated: list[Mod] = Field(alias="recentlyUpdated")


class GetFilesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Files%20Response"""

    data: list[File]


class GetGameResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Game%20Response"""

    data: Game


class GetGamesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Games%20Response"""

    data: list[Game]
    pagination: Pagination


class GetModFileResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Mod%20File%20Response"""

    data: File


class GetModFilesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Mod%20Files%20Response"""

    data: list[File]
    pagination: Pagination


class GetModResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Mod%20Response"""

    data: Mod


class GetModsResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Mods%20Response"""

    data: list[Mod]


class GetVersionTypesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Version%20Types%20Response"""

    data: list[GameVersionType]


class GetVersionsResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Versions%20Response%20-%20V1"""

    data: list[GameVersionsByType]


class GetVersionsResponse2(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Versions%20Response%20-%20V2"""

    data: list[GameVersionsByType]


class SearchModsResponse(BaseModel):
    data: list[Mod]
    pagination: Pagination


class ApiResponseOfListOfMinecraftGameVersion(BaseModel):
    """https://docs.curseforge.com/#tocS_ApiResponseOfListOfMinecraftGameVersion"""

    data: list[MinecraftGameVersion]


class StringResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_String%20Response"""

    data: str


class GetFingerprintMatchesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Fingerprint%20Matches%20Response"""

    data: list[FingerprintMatchesResult]


class GetFingerprintsFuzzyMatchesResponse(BaseModel):
    """https://docs.curseforge.com/#tocS_Get%20Fingerprints%20Fuzzy%20Matches%20Response"""

    data: list[FingerprintFuzzyMatchResult]
